@@ -4,6 +4,7 @@ use std::fmt;
 use std::ops;
 use std::f64;
 use std::u64;
+use std::iter::Sum;
 
 #[cfg(feature = "chrono")]
 use chrono;
@@ -32,7 +33,6 @@ pub const SECS_PER_DAY: f64 = SECS_PER_HOUR * 24.0;
 /// Number of seconds in a year.
 pub const SECS_PER_YEAR: f64 = SECS_PER_DAY * 365.0;
 
-
 /// A specific point in time.
 ///
 /// Types implementing `TimePoint` can have a `FloatDuration` computed between them
@@ -64,34 +64,42 @@ impl FloatDuration {
     ///
     /// `float_duration` considers one year to be exactly 365 days, with
     /// no consideration of leap years.
+    #[inline]
     pub fn years(years: f64) -> FloatDuration {
         FloatDuration { secs: years * SECS_PER_YEAR }
     }
     /// Create a new `FloatDuration` representing a number of days.
+    #[inline]
     pub fn days(days: f64) -> FloatDuration {
         FloatDuration { secs: days * SECS_PER_DAY }
     }
     /// Create a new `FloatDuration` representing a number of hours.
+    #[inline]
     pub fn hours(hours: f64) -> FloatDuration {
         FloatDuration { secs: hours * SECS_PER_HOUR }
     }
     /// Create a new `FloatDuration` representing a number of minutes.
+    #[inline]
     pub fn minutes(mins: f64) -> FloatDuration {
         FloatDuration { secs: mins * SECS_PER_MINUTE }
     }
     /// Create a new `FloatDuration` representing a number of seconds.
+    #[inline]
     pub fn seconds(secs: f64) -> FloatDuration {
         FloatDuration { secs: secs }
     }
     /// Create a new `FloatDuration` representing a number of milliseconds.
+    #[inline]
     pub fn milliseconds(millis: f64) -> FloatDuration {
         FloatDuration { secs: millis / MILLIS_PER_SEC }
     }
     /// Create a new `FloatDuration` representing a number of microseconds.
+    #[inline]
     pub fn microseconds(micros: f64) -> FloatDuration {
         FloatDuration { secs: micros / MICROS_PER_SEC }
     }
     /// Create a new `FloatDuration` representing a number of nanoseconds.
+    #[inline]
     pub fn nanoseconds(nanos: f64) -> FloatDuration {
         FloatDuration { secs: nanos / NANOS_PER_SEC }
     }
@@ -100,64 +108,79 @@ impl FloatDuration {
     ///
     /// `float_duration` considers one year to be exactly 365 days, with
     /// no consideration of leap years.
+    #[inline]
     pub fn as_years(&self) -> f64 {
         self.secs / SECS_PER_YEAR
     }
     /// Return the total number of fractional days represented by the `FloatDuration`.
+    #[inline]
     pub fn as_days(&self) -> f64 {
         self.secs / SECS_PER_DAY
     }
     /// Return the total number of fractional hours represented by the `FloatDuration`.
+    #[inline]
     pub fn as_hours(&self) -> f64 {
         self.secs / SECS_PER_HOUR
     }
     /// Return the total number of fractional minutes represented by the `FloatDuration`.
+    #[inline]
     pub fn as_minutes(&self) -> f64 {
         self.secs / SECS_PER_MINUTE
     }
     /// Return the total number of fractional seconds represented by the `FloatDuration`.
+    #[inline]
     pub fn as_seconds(&self) -> f64 {
         self.secs
     }
     /// Return the total number of fractional milliseconds represented by the `FloatDuration`.
+    #[inline]
     pub fn as_milliseconds(&self) -> f64 {
         self.secs * MILLIS_PER_SEC
     }
     /// Return the total number of fractional microseconds represented by the `FloatDuration`.
+    #[inline]
     pub fn as_microseconds(&self) -> f64 {
         self.secs * MICROS_PER_SEC
     }
     /// Return the total number of fractional nanoseconds represented by the `FloatDuration`.
+    #[inline]
     pub fn as_nanoseconds(&self) -> f64 {
         self.secs * NANOS_PER_SEC
     }
 
     /// Compute the absolute value of this duration.
+    #[inline]
     pub fn abs(self) -> FloatDuration {
         FloatDuration { secs: self.secs.abs() }
     }
     /// Return a new `FloatDuration` that represents zero elapsed time.
+    #[inline]
     pub fn zero() -> FloatDuration {
         FloatDuration { secs: 0.0 }
     }
     /// Returns true is this duration represents zero elapsed time (equals `FloatDuration::zero()`).
+    #[inline]
     pub fn is_zero(&self) -> bool {
         self.secs == 0.0
     }
     /// Returns true if the FloatDuration holds a positive amount of time.
+    #[inline]
     pub fn is_positive(&self) -> bool {
         self.secs.is_sign_positive()
     }
     /// Returns true if the FloatDuration holds a negative amount of time.
+    #[inline]
     pub fn is_negative(&self) -> bool {
         self.secs.is_sign_negative()
     }
 
     /// Return a new `FloatDuration` with the minimum possible value.
+    #[inline]
     pub fn min_value() -> FloatDuration {
         FloatDuration { secs: f64::MIN }
     }
     /// Return a new `FloatDuration` with the maximum possible value.
+    #[inline]
     pub fn max_value() -> FloatDuration {
         FloatDuration { secs: f64::MAX }
     }
@@ -185,6 +208,7 @@ impl FloatDuration {
     }
 
     /// Create a `FloatDuration` object from a `std::time::Duration`.
+    #[inline]
     pub fn from_std(duration: time::Duration) -> FloatDuration {
         FloatDuration::seconds((duration.as_secs() as f64) +
                                (duration.subsec_nanos() as f64) / NANOS_PER_SEC)
@@ -215,7 +239,8 @@ impl FloatDuration {
     /// precision if the duration is too large to be entirely represented as a single
     /// value. Thus, if the absolute value of the total number of nanoseconds is
     /// greater than `i64::MAX`, only millisecond precision will be captured.
-    pub fn from_chrono(duration: &chrono::Duration) -> FloatDuration {
+    #[inline]
+    pub fn from_chrono(duration: chrono::Duration) -> FloatDuration {
         if let Some(nanos) = duration.num_nanoseconds() {
             FloatDuration::nanoseconds(nanos as f64)
         } else {
@@ -268,46 +293,52 @@ impl<'de> Deserialize<'de> for FloatDuration {
 #[cfg(feature = "chrono")]
 impl<Tz: chrono::TimeZone> TimePoint for chrono::DateTime<Tz> {
     type Error = ();
+    #[inline]
     fn float_duration_since(self, since: chrono::DateTime<Tz>) -> Result<FloatDuration, ()> {
         let chrono_duration = self.signed_duration_since(since);
-        Ok(FloatDuration::from_chrono(&chrono_duration))
+        Ok(FloatDuration::from_chrono(chrono_duration))
     }
 }
 #[cfg(feature = "chrono")]
 impl<Tz: chrono::TimeZone> TimePoint for chrono::Date<Tz> {
     type Error = ();
+    #[inline]
     fn float_duration_since(self, since: chrono::Date<Tz>) -> Result<FloatDuration, ()> {
         let chrono_duration = self.signed_duration_since(since);
-        Ok(FloatDuration::from_chrono(&chrono_duration))
+        Ok(FloatDuration::from_chrono(chrono_duration))
     }
 }
 #[cfg(feature = "chrono")]
 impl TimePoint for chrono::NaiveDate {
     type Error = ();
+    #[inline]
     fn float_duration_since(self, since: chrono::NaiveDate) -> Result<FloatDuration, ()> {
         let chrono_duration = self.signed_duration_since(since);
-        Ok(FloatDuration::from_chrono(&chrono_duration))
+        Ok(FloatDuration::from_chrono(chrono_duration))
     }
 }
 #[cfg(feature = "chrono")]
 impl TimePoint for chrono::NaiveTime {
     type Error = ();
+    #[inline]
     fn float_duration_since(self, since: chrono::NaiveTime) -> Result<FloatDuration, ()> {
         let chrono_duration = self.signed_duration_since(since);
-        Ok(FloatDuration::from_chrono(&chrono_duration))
+        Ok(FloatDuration::from_chrono(chrono_duration))
     }
 }
 #[cfg(feature = "chrono")]
 impl TimePoint for chrono::NaiveDateTime {
     type Error = ();
+    #[inline]
     fn float_duration_since(self, since: chrono::NaiveDateTime) -> Result<FloatDuration, ()> {
         let chrono_duration = self.signed_duration_since(since);
-        Ok(FloatDuration::from_chrono(&chrono_duration))
+        Ok(FloatDuration::from_chrono(chrono_duration))
     }
 }
 
 impl TimePoint for time::Instant {
     type Error = ();
+    #[inline]
     fn float_duration_since(self, since: time::Instant) -> Result<FloatDuration, ()> {
         let std_duration = self.duration_since(since);
         Ok(FloatDuration::from_std(std_duration))
@@ -315,6 +346,7 @@ impl TimePoint for time::Instant {
 }
 impl TimePoint for time::SystemTime {
     type Error = time::SystemTimeError;
+    #[inline]
     fn float_duration_since(self,
                             since: time::SystemTime)
                             -> Result<FloatDuration, time::SystemTimeError> {
@@ -423,6 +455,20 @@ impl Default for FloatDuration {
         FloatDuration::zero()
     }
 }
+impl Sum for FloatDuration {
+    fn sum<I>(iter: I) -> FloatDuration
+        where I: Iterator<Item = FloatDuration>
+    {
+        iter.fold(FloatDuration::zero(), |a, b| a + b)
+    }
+}
+impl<'a> Sum<&'a FloatDuration> for FloatDuration {
+    fn sum<I>(iter: I) -> FloatDuration
+        where I: Iterator<Item = &'a FloatDuration>
+    {
+        iter.fold(FloatDuration::zero(), |a, &b| a + b)
+    }
+}
 
 #[cfg(feature = "approx")]
 impl ApproxEq for FloatDuration {
@@ -519,6 +565,13 @@ mod tests {
 
         assert_eq!(FloatDuration::seconds(10.0) - FloatDuration::minutes(1.0),
                    FloatDuration::seconds(-50.0));
+
+        let inf = FloatDuration::seconds(10.0) / 0.0;
+
+        assert!(inf.as_seconds().is_infinite());
+        assert!(inf.as_years().is_infinite());
+        assert!(inf.as_microseconds().is_infinite());
+        assert!(FloatDuration::hours(10.0) / FloatDuration::minutes(0.0) == f64::INFINITY);
     }
 
     #[test]
@@ -572,22 +625,38 @@ mod tests {
         assert_eq!(format!("{}", FloatDuration::years(2.5)), "2.5 years");
     }
 
+    #[test]
+    fn test_sum() {
+        let zero: [FloatDuration; 0] = [];
+
+        assert_eq!(zero.iter().sum::<FloatDuration>(), FloatDuration::zero());
+
+        assert_eq!([FloatDuration::milliseconds(50.0),
+                    FloatDuration::milliseconds(30.0),
+                    FloatDuration::zero()]
+                       .iter()
+                       .sum::<FloatDuration>(),
+                   FloatDuration::milliseconds(80.0));
+        assert_eq!([FloatDuration::days(2.0)].iter().sum::<FloatDuration>(),
+                   FloatDuration::days(2.0));
+    }
+
     #[cfg(feature = "chrono")]
     #[test]
     fn test_chrono_conversion() {
-        assert_eq!(FloatDuration::from_chrono(&chrono::Duration::minutes(10)),
+        assert_eq!(FloatDuration::from_chrono(chrono::Duration::minutes(10)),
                    FloatDuration::minutes(10.0));
-        assert_eq!(FloatDuration::from_chrono(&chrono::Duration::hours(72)),
+        assert_eq!(FloatDuration::from_chrono(chrono::Duration::hours(72)),
                    FloatDuration::days(3.0));
-        assert_eq!(FloatDuration::from_chrono(&chrono::Duration::nanoseconds(500)),
+        assert_eq!(FloatDuration::from_chrono(chrono::Duration::nanoseconds(500)),
                    FloatDuration::nanoseconds(500.0));
-        assert_eq!(FloatDuration::from_chrono(&chrono::Duration::microseconds(-20000)),
+        assert_eq!(FloatDuration::from_chrono(chrono::Duration::microseconds(-20000)),
                    FloatDuration::milliseconds(-20.0));
-        assert_eq!(FloatDuration::from_chrono(&chrono::Duration::zero()),
+        assert_eq!(FloatDuration::from_chrono(chrono::Duration::zero()),
                    FloatDuration::zero());
-        assert_eq!(FloatDuration::from_chrono(&chrono::Duration::hours(10000)),
+        assert_eq!(FloatDuration::from_chrono(chrono::Duration::hours(10000)),
                    FloatDuration::hours(10000.0));
-        assert_eq!(FloatDuration::from_chrono(&chrono::Duration::milliseconds(1i64 << 62)),
+        assert_eq!(FloatDuration::from_chrono(chrono::Duration::milliseconds(1i64 << 62)),
                    FloatDuration::milliseconds((1i64 << 62) as f64));
 
         assert_eq!(FloatDuration::minutes(2.5).to_chrono().unwrap(),
