@@ -23,7 +23,7 @@
 //!
 //! ```ignore
 //! [dependencies]
-//! float_duration = "0.2"
+//! float_duration = "0.3"
 //! ```
 //! # Overview
 //!
@@ -84,6 +84,20 @@
 //! }
 //! ```
 //!
+//! Run a function on 100 evenly spaced durations:
+//!
+//! ```rust
+//! use float_duration::{FloatDuration, subdivide};
+//!
+//! fn compute_value(t: FloatDuration) -> f64 {
+//!     t.as_seconds()*t.as_seconds()
+//! }
+//!
+//! for time in subdivide(FloatDuration::zero(), FloatDuration::hours(1.0), 100) {
+//!     println!("{}", compute_value(time));
+//! }
+//! ```
+//!
 //! # Library Support
 //!
 //! Currently `float_duration` can be compiled without any dependencies, but it
@@ -157,6 +171,8 @@ extern crate serde_test;
 
 pub mod duration;
 pub mod error;
+pub mod iter;
 
 pub use duration::{FloatDuration, TimePoint};
 pub use error::OutOfRangeError;
+pub use iter::{subdivide, subdivide_with_step};
